/*
 * file CcAttributeType.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * com.ibm.rational.wvcm.stp.cc.CcAttributeType
 * 
 * (C) Copyright IBM Corporation 2004, 2013.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stpex.StpExEnumeration;


/**
 * <p>A proxy for a ClearCase attribute type.
 * </p>
 * <p>The ClearCase attribute mechanism lets you associate arbitrary
 * name/value pairs with particular ClearCase versions or elements.
 * </p>
 * <p>Attributes are typically used to customize and extend standard
 * ClearCase functionality.  For example, to notify various people in your group
 * when particular files are modified, you could store those peoples' email
 * addresses in an "EmailList" attribute associated with each element of interest.
 * Then associate a checkin "post-op" trigger with those
 * elements that reads the attribute value and sends the notice.
 * </p>
 * <p>You can also use the cleartool "find" command to find elements and
 * versions based on their attributes.
 * </p>
 * <p>For more attribute information, see the ClearCase "Guide to Managing Software Projects"
 * manual, and the cleartool man pages "mkattype" and "find".
 * </p>
 */
public interface CcAttributeType extends CcTypeBase
{
    /**
     * Value type of a ClearCase attribute type.
     * The value type determines helps constrain the legal values and format of an
     * attribute instance.  
     */
    enum ValueType implements StpExEnumeration
    {
        /** Value is an integer. */
        INTEGER,

        /** Value is floating-point. */
        REAL,
        
        /** Value is a string. */
        STRING,
        
        /** Value represents a time. */
        TIME,
        
        /** Value is an opaque arbitrary byte sequences. */
        OPAQUE;
    }
    
    /**
     * The default value for this attribute type.  It is aways provided in
     * string form, regardless of its value type.  If no default value
     * has been set, the property will be <code>null</code>. 
     */
    PropertyName<String> DEFAULT_VALUE =
            new PropertyName<String>(PROPERTY_NAMESPACE, "default-value");
    
    /**
     * Get the value of this proxy's {@link #DEFAULT_VALUE} property.
     * @return this attribute type's default value, or <code>null</code> if not defined
     */
    String getDefaultValue() throws WvcmException;

    /**
     * The value type of this attribute type
     */
    PropertyName<ValueType> VALUE_TYPE =
            new PropertyName<ValueType>(PROPERTY_NAMESPACE, "value-type");

    /**
     * Get the value of this proxy's {@link #VALUE_TYPE} property.
     * @return this attribute type's value type
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ValueType getValueType() throws WvcmException;


}
